import axios from 'axios'
import {appConfig} from "./config"
import * as qs from "qs"
import dayjs from "dayjs"
import {ICourse, IJsonCourse, TChannel} from "./types"
import {JsonCourseList} from "./poll"
import axiosRetry from "axios-retry"

let sduAxios = axios.create({
    headers: {
        'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8',
        'Cookie': appConfig.cookie,
        'X-Requested-With': 'XMLHttpRequest',
        'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:92.0) Gecko/20100101 Firefox/92.0'
    },
    baseURL: 'https://bkzhjx.wh.sdu.edu.cn/',
    timeout: 5000,
    transformResponse: data => {
        return JSON.parse(data)
    }
    // maxRedirects: 0
})

axiosRetry(sduAxios, {
    retries: 2,
    shouldResetTimeout: true,
    retryCondition: (_error) => true,
})

export async function logAndNotifyUser(text: string) {
    console.log(text)
    await axios.post('https://qmsg.zendee.cn/send/' + appConfig.qmsgKey, qs.stringify({
        msg: text
    }), {
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded'
        }
    })
    // await axios.post('https://' + appConfig.tgApiDomain + '/bot' + appConfig.tgApiKey + '/sendMessage', qs.stringify({
    //     chat_id: appConfig.tgChatId,
    //     text: text
    // }))
}

export function sleep(ms: number) {
    return new Promise(resolve => setTimeout(resolve, ms))
}

export function getTimeNow() {
    return dayjs().format('hh:mm:ss A')
}

export function findJsonCourse(course: ICourse): IJsonCourse | undefined {
    return JsonCourseList[course.channel].find(single => single.kch == course.kch && parseInt(single.kxh) == course.kxh)
}

export async function acquireCourse(course: IJsonCourse, channel: TChannel): Promise<boolean> {
    let segment = ''
    switch (channel) {
        case 'bx':
            segment = 'bxxkOper'
            break
        case 'xx':
            segment = 'xxxkOper'
            break
        case 'rx':
            segment = 'ggxxkxkOper'
            break
    }
    try {
        let resp = await sduAxios.get('/jsxsd/xsxkkc/' + segment + '?kcid=' + course.jx02id + '&cfbs=null&jx0404id=' + course.jx0404id + '&xkzy=&trjf=')
        if (resp.data.success === true) {
            return true
        } else {
            logAndNotifyUser(resp.data.message)
            return false
        }
    } catch (e) {
        return false
    }
}

export async function exitCourse(course: IJsonCourse): Promise<boolean> {
    try {
        let resp = await sduAxios.get('/jsxsd/xsxkjg/xstkOper?jx0404id=' + course.jx0404id)
        if (resp.data.success === true) {
            return true
        } else {
            logAndNotifyUser('[exitCourse]退课失败：' + resp.data.message)
            return false
        }
    } catch (e) {
        logAndNotifyUser('[exitCourse]网络异常，退课失败')
        return false
    }
}

export {sduAxios}
